@@ -10,6 +10,7 @@ from django.utils.http import http_date, parse_http_date, parse_etags, quote_eta
 from django.utils.cache import patch_cache_control
 from .http import codes, methods
 from .serializers import serializers
+from .mixins import TemplateResponseMixin
 
 EPOCH_DATE = datetime(1970, 1, 1, 0, 0, 0)
 MAX_CACHE_AGE = 60 * 60 * 24 * 30
@@ -18,6 +19,16 @@ MAX_CACHE_AGE = 60 * 60 * 24 * 30
 usable = lambda x, y: callable(getattr(x, y, None))
 
 
+def no_content_response(response):
+    "Cautious assessment of the response body for no content."
+    if response._container is None:
+        return True
+    if isinstance(response._container, (list, tuple)):
+        if len(response._container) == 1 and response._container[0] == '':
+            return True
+    return False
+
+
 class UncacheableResponse(HttpResponse):
     "Response class that will never be cached."
     def __init__(self, *args, **kwargs):
@@ -224,10 +235,22 @@ class Resource(object):
     # - http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9.4
     cache_must_revalidate = False
 
+    def __init__(self, **kwargs):
+        for key in kwargs:
+            # Not methods nor methods
+            if key in self.allowed_methods or callable(getattr(self, key, None)):
+                raise TypeError('No HTTP handlers nor methods can be overriden.')
+            if not hasattr(self, key):
+                raise TypeError('{0} is not a valid keyword argument for this resource.'.format(key))
+            setattr(self, key, kwargs[key])
+
     # ## Initialize Once, Process Many
     # Every `Resource` class can be initialized once since they are stateless
     # (and thus thread-safe).
     def __call__(self, request, *args, **kwargs):
+        return self.dispatch(request, *args, **kwargs)
+
+    def dispatch(self, request, *args, **kwargs):
         # Process the request. This includes all the necessary checks prior to
         # actually interfacing with the resource itself.
         response = self.process_request(request, *args, **kwargs)
@@ -238,11 +261,30 @@ class Resource(object):
             method_handler = getattr(self, request.method.lower())
             response = method_handler(request, *args, **kwargs)
 
+            # If the return value of the handler is not a response, pass
+            # the return value into the render method.
+            if not isinstance(response, HttpResponse):
+                response = self.render(request, response, *args, **kwargs)
+
         # Process the response, check if the response is overridden and
         # use that instead.
-        # TODO not sure if this is sound for a simple resource
         return self.process_response(request, response)
 
+    def render(self, request, content, *args, **kwargs):
+        "Renders the response based on the content returned from the handler."
+        response = HttpResponse()
+
+        if request.method != methods.HEAD:
+            if isinstance(content, (basestring, file)):
+                response.content = content
+            else:
+                accept_type = getattr(request, '_accept_type', None)
+                if accept_type and serializers.supports_encoding(accept_type):
+                    response.content = serializers.encode(accept_type, content)
+                response['Content-Type'] = accept_type
+
+        return response
+
     # ## Request Method Handlers
     # ### _HEAD_ Request Handler
     # Default handler for _HEAD_ requests. For this to be available,
@@ -308,7 +350,8 @@ class Resource(object):
     # ### Request Entity Too Large
     # Check if the request entity is too large to process.
     def is_request_entity_too_large(self, request, response, *args, **kwargs):
-        if request.META['CONTENT_LENGTH'] > self.max_request_entity_length:
+        if self.max_request_entity_length and \
+            request.META['CONTENT_LENGTH'] > self.max_request_entity_length:
             return True
 
     # ### Method Not Allowed
@@ -359,10 +402,15 @@ class Resource(object):
     # ### Precondition Required
     # Check if a conditional request is
     def is_precondition_required(self, request, response, *args, **kwargs):
+        if not self.require_conditional_request:
+            return False
+
         if self.use_etags and 'HTTP_IF_MATCH' not in request.META:
             return True
+
         if self.use_last_modified and 'HTTP_IF_UNMODIFIED_SINCE' not in request.META:
             return True
+
         return False
 
     def is_precondition_failed(self, request, response, *args, **kwargs):
@@ -387,7 +435,6 @@ class Resource(object):
 
         return False
 
-
     # ### Not Found
     # Checks if the requested resource exists.
     def is_not_found(self, request, response, *args, **kwargs):
@@ -399,7 +446,6 @@ class Resource(object):
         return False
 
 
-
     # ## Request Accept-* handlers
 
     # Checks if the requested `Accept` mimetype is supported. Defaults
@@ -518,17 +564,6 @@ class Resource(object):
         return True
 
     # Utility methods
-    def write(self, request, response, content, accept_type=None):
-        if content is None:
-            response.status_code = codes.no_content
-        elif isinstance(content, basestring) or isinstance(content, file):
-            response.content = content
-        else:
-            accept_type = accept_type or getattr(request, '_accept_type', None)
-            if accept_type and serializers.supports_encoding(accept_type):
-                response.content = serializers.encode(accept_type, content)
-            response['Content-Type'] = accept_type
-
     def get_cache(self, request, response):
         "Returns the cache to be used for various components."
         from django.core.cache import cache
@@ -614,6 +649,17 @@ class Resource(object):
                 response.status_code = codes.too_many_requests
                 return response
 
+        # ### 405 Method Not Allowed
+        if self.is_method_not_allowed(request, response, *args, **kwargs):
+            response.status_code = codes.method_not_allowed
+            return response
+
+        # ### 406 Not Acceptable
+        # Checks Accept and Accept-* headers
+        if self.is_not_acceptable(request, response, *args, **kwargs):
+            response.status_code = codes.not_acceptable
+            return response
+
         # ### Process an _OPTIONS_ request
         # Enough processing has been performed to allow an OPTIONS request.
         if request.method == methods.OPTIONS and 'OPTIONS' in self.allowed_methods:
@@ -631,21 +677,9 @@ class Resource(object):
 
             # ### 413 Request Entity Too Large
             # Check if the entity is too large for processing
-            if self.max_request_entity_length:
-                if self.is_request_entity_too_large(request, response, *args, **kwargs):
-                    response.status_code = codes.request_entity_too_large
-                    return response
-
-        # ### 405 Method Not Allowed
-        if self.is_method_not_allowed(request, response, *args, **kwargs):
-            response.status_code = codes.method_not_allowed
-            return response
-
-        # ### 406 Not Acceptable
-        # Checks Accept and Accept-* headers
-        if self.is_not_acceptable(request, response, *args, **kwargs):
-            response.status_code = codes.not_acceptable
-            return response
+            if self.is_request_entity_too_large(request, response, *args, **kwargs):
+                response.status_code = codes.request_entity_too_large
+                return response
 
         # ### 404 Not Found
         # Check if this resource exists. Note, if this requires a database
@@ -671,10 +705,9 @@ class Resource(object):
         # Prevents the "lost udpate" problem and requires client to confirm
         # the state of the resource has not changed since the last `GET`
         # request. This applies to `PUT` and `PATCH` requests.
-        if self.require_conditional_request:
-            if request.method == methods.PUT or request.method == methods.PATCH:
-                if self.is_precondition_required(request, response, *args, **kwargs):
-                    return UncacheableResponse(status=codes.precondition_required)
+        if request.method == methods.PUT or request.method == methods.PATCH:
+            if self.is_precondition_required(request, response, *args, **kwargs):
+                return UncacheableResponse(status=codes.precondition_required)
 
         # ### 412 Precondition Failed
         # Conditional requests applies to GET, HEAD, PUT, and PATCH.
@@ -724,16 +757,14 @@ class Resource(object):
     # ## Process the normal response returned by the handler
     def process_response(self, request, response):
         if not isinstance(response, HttpResponse):
-            content = response
-            response = HttpResponse()
-
-            if request.method != methods.HEAD:
-                self.write(request, response, content)
+            response = HttpResponse(response)
 
         if request.method == methods.HEAD:
             response.content = ''
+        elif response.status_code == codes.ok and no_content_response(response):
+            response.status_code = codes.no_content
 
-        if request.method == methods.GET or request.method == methods.HEAD:
+        if request.method in (methods.GET, methods.HEAD):
             self.response_cache_control(request, response)
 
             if self.use_etags:
@@ -744,8 +775,6 @@ class Resource(object):
 
         return response
 
-    # ## Request Programatically
-    # For composite resources, `resource.apply` can be used on related resources
-    # with the original `request`.
-    def apply(self, request, *args, **kargs):
-        pass
+
+class TemplateResource(TemplateResponseMixin, Resource):
+    pass
